<?php
/**
* Contains the class SkroutzAPI 
*/
/**
* Implements querying the Skroutz API v.2b via cURL and JSON
*
* The SkroutzAPI class can be either used normally or following the singleton pattern.
*
* @author Nick Andriopoulos <nand@hexblot.gr>
* @copyright Nick Andriopoulos 2011
* @link http://www.hexblot.gr/projects/skroutz
* @version 0.9b
*/
class SkroutzAPI {
	const SKDOMAIN = 'apiv2.skroutz.gr';  	// Change this if instructed by Skroutz
	private static $auth;                 	// Auth token cache
	private static $cache;				  	// Action cache
	private static $instance;				// Singleton instance
	private $api_key;                     	// As provided by Skroutz
	private $debug;                       	// Debug level - 
											//   5=trace, 0=normal, <0=error
	// Class constructor
	public function __construct($key) {
		$this->debug   = 0;     // Change as you feel necessary, 0 is production
		$this->api_key = $key;  // Cannot proceed without this
		self:$instance = -1;	// Disable singleton
		// Check that cURL and JSON (or a workaround for theris functionality) are present
		if ( function_exists('curl_init') && function_exists('json_decode') ) {
			return true;
		} else {
			return false;
		}
	}

	public static function singleton($key) {
		if (!isset(self::$instance)) {
			$c= __CLASS__;
			self::$instance = new $c($key);
		} elseif (self::$instance==-1) {
			trigger_error('Cannot use singleton after normal initialization', E_USER_ERROR);
		}		
		return self::$instance;	
	}
	
	// Get a product via its SKU
	public function get_product_by_sku($sku, $fields = "") {

		if ( !is_numeric($sku) ) {
			return $this->error('Must provide a numeric SKU');
		}

		$result = $this->_call("sku/$sku");
		return $this->proc_result($result, $fields);
	}

	// Get a product via alternate methods
	public function get_product_by_other($code, $type, $fields="") {
		switch($type) {
			case 'shop_sku':
				$action = 'sku/lookup/shop/'.$code['shop_id'].'/sku/'.$code['sku'];
				break;
			case 'part_no':
				$action = 'sku/lookup/pn/'.$code;
				break;
			case 'barcode':
				$action = 'sku/lookup/barcode/'.$barcode;
				break;
			default:
				return $this->error("Unknown fetch type: $type, use one of 'shop_sku','part_no', 'barcode'");
		}
		
		$result = $this->_call($action);
		return $this->proc_result($result, $fields);	
	
	}
	
	// Avoid using this for anything other than debug!
	// Make a query directly
	public function raw_q($action, $params) {
		if(isset($params['access_token'])) unset $params['access_token'];
		return $this->_call($action, $params);    
	}

	public function auth() {
		$skh = $this->_call('authenticate', array('api_key'=>$this->api_key));
		if ( is_array($skh['error']) ) {
			// Error authenticating
			$this->log("Authentication to Skroutz failed: ".$skh['error']['text'],-1);
			return $skh;
		} else {
			// Cache the credentials and expiry date
			$this->auth['token']   = $skh['result']['access_token'];    
			$this->auth['expires'] = strtotime ($skh['result']['access_token_expiry']);
			$this->log("Got new auth token, now in cache",3);
		}
	}
	
	// Workhorse function
	private function _call($action, $params="") {
		$this->log("Entering _call with action: $action and params: ".print_r($params,1),5);
		// Authentication happens only over HTTPS
		if ( $action == 'authenticate' ) {
			$base_path = "https://".self::SKDOMAIN.'/json/';  
		} else {
			$base_path = "http://" .self::SKDOMAIN.'/json/';    
		}
	  
		// Check to see if there's a valid cached auth token
		if ( $action != 'authenticate' && (!is_array(self::$auth) || ( self::$auth['expires'] - time() )<0 )) {
			// Initial call, query Sk to get an auth token
			$this->log("No cached auth found, recursing", 5);
			$this->auth();		
		} 
	  
		if ( $action != 'authenticate' ) {
			// If this is not an auth call, we need to provide an access token.
			//  (It should be impossible to be here without a token for any action other than authenticate
			//  since one is generated above)
			$this->log("Inserting auth token to request", 5);
			$params['access_token'] = $this->auth['token']);
		}

		// Initialize cURL
		$this->log("Initializing curl",3);
		$curl_handle = curl_init();

		curl_setopt($curl_handle, CURLOPT_URL           , $base_path.$action.'/'    );
		curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, 1                         );
		curl_setopt($curl_handle, CURLOPT_POSTFIELDS    , $params                   );
		curl_setopt($curl_handle, CURLOPT_ENCODING      , 'gzip'                    );
		curl_setopt($curl_handle, CURLOPT_HTTPHEADER    , array('Connection: close'));
		// Normally cURL outputs an 'Expect: 100-continue' header that breaks Sk, jam it
		curl_setopt($curl_handle, CURLOPT_HTTPHEADER    , array('Expect:'          ));

		$buffer = curl_exec($curl_handle);
		curl_close($curl_handle);

		if (empty($buffer)) {
			// Error for unknown reason, most likely network related
			$this->log("Unknown cURL error, got empty response",-1);
			$res = $this->error('Empty data from cURL');
		} else {
			$this->log("Received response :".$buffer, 6);
			// Make it into an associative array
			$res = json_decode( $buffer, true );
		}
		return $res;
	}

	// Basic logger with variable levels  
	private function log($msg, $lvl=0) {
	  if( $lvl <= $this->debug ) {
	   drupal_set_message($msg);
	  }      
	}
	
	private function error($msg = "Class error") {
	
	private function proc_result($result, $fields) {
		if( is_array($result) && !is_array($result['error']) ) {
			// If fields array is set, return only those elements.
			if ( $fields != "" ) {
				return array_intersect_key($result['result']['sku'], array_fill_keys($fields,0));
			} else {
				return $result['result']['sku'];
			}
		} else {
			// Return Skroutz error array
			return $result;
		}
	}
		
			'error' => array(
				'number' => -1,
				'code'	=> 'Undefined',
				'text'	=> $msg
			)
		);
	}
}

